//
//  ViewController.swift
//  ContextClues
//
//  Created by Jake Marsh on 6/1/15.
//  Copyright (c) 2015 Magnus. All rights reserved.
//

import UIKit

// IMPORTANT NOTE: This code to call the API is *absolutely hideous* I know. Daily content can be challenging,
// please excuse any rough edges in these sample projects. :)
//
// In production code, you'd want to make all of these bits that interact with and parse data retrieved from a web
// service in MUCH better ways. Future bites will likely cover best practices around these sorts of things.
//
// - Jake

let WolframAlphaAppID = "3PK6A8-E94VR759XR"

class WolframAlphaAPI : NSObject {
	let baseURL = "http://api.wolframalpha.com/v2"
	var currentTask: NSURLSessionDataTask?

	func search(query: String, completion: ((results: AnyObject?, error: NSError?) -> ())?) {
		if let t = currentTask { t.cancel(); currentTask = nil }

		let escapedQuery = query.stringByAddingPercentEscapesUsingEncoding(NSUTF8StringEncoding)!

		let URL = NSURL(string: "\(baseURL)/query?appid=\(WolframAlphaAppID)&input=\(escapedQuery)")!

		println("Requesting: \(URL)")

		currentTask = NSURLSession.sharedSession().dataTaskWithURL(URL) {(data, response, error) in
			dispatch_async(dispatch_get_main_queue()) {
				if let c = completion, d = data  {
					let xml = SWXMLHash.parse(d)

					println(xml)

					for pod in xml["queryresult"][0]["pod"].all {
						if let podID = pod.element?.attributes["position"] {
							if (podID as NSString).hasPrefix("200") {
								if let thingDescription = pod["subpod"]["plaintext"].element?.text {
									c(results: thingDescription, error: nil)
								}

								return
							}
						}
					}

					c(results: nil, error: error)
				}
			}
		}

		currentTask!.resume()
	}
}

class ViewController: UIViewController {
	@IBOutlet weak var inputTextView: UITextView!
	@IBOutlet weak var outputTextView: UITextView!
	
	var properNouns: [String] = []
	let API = WolframAlphaAPI()

	override func viewDidLoad() {
		super.viewDidLoad()

		updateProperNouns()
	}

	func updateProperNouns() {
		let options: NSLinguisticTaggerOptions = .OmitWhitespace | .OmitPunctuation | .JoinNames
		let schemes = NSLinguisticTagger.availableTagSchemesForLanguage("en")
		let tagger = NSLinguisticTagger(tagSchemes: schemes, options: Int(options.rawValue))

		tagger.string = inputTextView.text

		var results = ""
		properNouns = []

		tagger.enumerateTagsInRange(NSMakeRange(0, (inputTextView.text as NSString).length), scheme: NSLinguisticTagSchemeNameType, options: options) { (tag, tokenRange, sentenceRange, _) in
			let token = (self.inputTextView.text as NSString).substringWithRange(tokenRange)

			results += "\(token): \(tag)\n"

			if contains([NSLinguisticTagPersonalName, NSLinguisticTagPlaceName, NSLinguisticTagOrganizationName], tag) {
				self.properNouns.append(token)
			}
		}

		if let query = properNouns.first {
			API.search(query) { (thingDescription, error) in
				if let d = thingDescription as? String {
					results = d + "\n\n" + results
				}

				self.outputTextView.text = results
			}
		}
	}
}

extension ViewController : UITextViewDelegate {
	func textViewDidChange(textView: UITextView) {
		updateProperNouns()
	}
}
